use std::fs;

use pest::Parser;
use pest_derive::Parser;

#[derive(Parser)]
#[grammar = "rpy.pest"]
struct RpyParser;

#[derive(Debug)]
pub enum Token {
    Keyword(String),
    String(String),
    Array(Vec<Token>),
}

pub fn parse(script: &str) -> Vec<Vec<Token>> {
    let file = RpyParser::parse(Rule::file, script)
        .expect("unsuccessful parse")
        .next().unwrap();
    let mut lines = Vec::new();
    for line in file.into_inner() {
        let mut tokens = Vec::new();
        match line.as_rule() {
            Rule::line => {
                for token in line.into_inner() {
                    tokens.push(parse_token(token));
                }
            },
            Rule::EOI => (),
            _ => unreachable!(),
        }
        lines.push(tokens);
    } 
    lines
}

fn parse_token(pair: pest::iterators::Pair<Rule>) -> Token {
    let token = pair.as_rule();
    match token {
        Rule::token => {},
        _ => panic!("Not a token!"),
    };
    let contents = pair.into_inner().next().unwrap();
    let contents_rule = contents.as_rule();
    match contents_rule {
        Rule::string => {
            let data = contents.into_inner().next().unwrap();
            Token::String(match data.as_rule() {
                Rule::single_quote_string_data => data.as_str().replace("\\'", "'"),
                Rule::double_quote_string_data => data.as_str().replace("\\\"", "\""),
                _ => unreachable!(),
            })
        },
        Rule::array => {
            let mut array = Vec::new();
            for token in contents.into_inner() {
                array.push(parse_token(token));
            }
            Token::Array(array)
        }
        Rule::keyword => Token::Keyword(contents.as_str().to_owned()),
        __ => unreachable!(),
    }
}

pub fn parse_file(file_path: &str) -> Vec<Vec<Token>> {
    let unparsed_file = fs::read_to_string(file_path).expect("cannot find file");
    parse(&unparsed_file)
}
